import java.applet.Applet;

import java.awt.Choice;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
   This class demonstrates some of the geometric figures you
   can draw with a Graphics object.  Egregious use of colors
   is avoided.
*/
public class DrawingDemoApplet extends Applet
{
  /**
     Called by the browser.  Initializes the GUI.
  */
  public void init()
  {
    // Add a Choice component.  Selecting one of the
    // categories of geometric figures will draw some
    // examples from that set.
    //
    Choice choice = new Choice();
    choice.add("Lines");
    choice.add("Ovals");
    choice.add("Polygons");
    choice.add("Rectangles");
    choice.add("Round Rectangles");
    choice.add("Arcs");
    choice.add("Clear");

    add(choice);

    // Create and add a listener for the Choice selection.
    // Set the selection in the instance field.  The paint()
    // method reads the field and calls a helper method to
    // paint the correct geometric figures.
    //
    ItemListener iL = new ItemListener()
      {
	public void itemStateChanged(ItemEvent e)
	  {
	    selection = ((String) e.getItem());
	    repaint();
	  }
      };
    choice.addItemListener(iL);
  }


  /**
     Paints the applet accordingly.  This method overrides
     the paint method in Applet.  We don't need to dispose
     of the graphics object; the caller will do it.

     <p>Checks the selection made last and calls the
     appropriate method to paint the correct geometric
     shapes.

     @param g the graphics context in which to draw.  
  */
  public void paint(Graphics g)
  {
    Rectangle r = g.getClipBounds();
    System.out.println("Clip Rectangle: x = " + r.x +
		       "; y = " + r.y +
		       "; width  = " + r.width +
		       "; height = " + r.height);

//     g.setClip(0, 0, getSize().width, getSize().height);
    g.clearRect(0, 0, getSize().width, getSize().height);

    if (selection == "Lines")
      drawLines(g);
    else if(selection == "Ovals")
      drawOvals(g);
    else if(selection == "Polygons")
      drawPolygons(g);
    else if(selection == "Rectangles")
      drawRectangles(g);
    else if(selection == "Round Rectangles")
      drawRoundRectangles(g);
    else if (selection == "Arcs")
      drawArcs(g);
    else if (selection == "Clear")
      g.clearRect(0, 0, getSize().width, getSize().height);

//     g.setClip(r);
//     g.drawRect(r.x, r.y, r.width - 2, r.height - 2);
//     g.drawRect(r.x + 1, r.y + 1, r.width - 4, r.height - 4);
  }


  /**
     Draw some sample lines.  For all the drawing routines
     the Graphics passed is the same graphics passed to the
     paint() method.  A new one is not created.

     @param g a reference to the graphics in which to draw.  
  */
  private void drawLines(Graphics g)
  {
    // You can effectively create lines thicker than one pixel by
    // drawing adjacent pixels.  The following six calls draw in
    // adjacent rows of pixels.
    //
    g.drawLine(50, 50, 280, 280);
    g.drawLine(50, 51, 280, 281);
    g.drawLine(50, 52, 280, 282);
    g.drawLine(50, 53, 280, 283);
    g.drawLine(50, 54, 280, 284);
    g.drawLine(50, 55, 280, 285);

    // The following three calls creates a line of thickness
    // 3 pixels.
    //
    g.drawLine(50, 75, 280, 305);
    g.drawLine(50, 76, 280, 306);
    g.drawLine(50, 77, 280, 307);

    g.drawLine(50, 100, 280, 330);
  }


  private void drawOvals(Graphics g)
  {
    g.drawOval(75, 100, 200, 125);
    g.drawOval(100, 125, 150, 100);
    g.fillOval(125, 150, 100, 75);
  }


  private void drawPolygons(Graphics g)
  {
    // A set of (x, y) points that are connected with straight
    // lines to form the polygon.  The polygon is automatically
    // closed if the start and end points are not the same.
    // 
    int [] xPoints = {75, 150, 175, 200, 275, 210, 240, 175, 
		      115, 140};
    int [] yPoints = {150, 150, 75, 150, 150, 185, 260, 210, 
		      260, 185};

    g.drawPolygon(xPoints, yPoints, 10);
  }


  private void drawRectangles(Graphics g)
  {
    g.drawRect(50, 100, 250, 150);
    g.draw3DRect(75, 125, 200, 100, true);
    g.fill3DRect(125, 160, 100, 25, true);
  }


  private void drawRoundRectangles(Graphics g)
  {
    // The arc width and height is the same as the width and
    // height of the bounding rectangle.  The sides of the
    // oval look smooth.
    //
    g.drawRoundRect(20, 50, 50, 125, 50, 125);

    // The bounding rectangle is a square, so this draws a
    // circle.
    //
    g.setColor(Color.magenta);
    g.fillRoundRect(100, 50, 50, 50, 50, 50);

    g.setColor(Color.black);

    // The arc width and height are less than the width and
    // height of the bounding rectangle.  The sides of the
    // oval will be compressed.
    //
    g.drawRoundRect(100, 125, 200, 90, 150, 75);

    // No compression of the arcs will be done here.
    //
    g.drawRoundRect(50, 250, 250, 75, 250, 75);
  }


  private void drawArcs(Graphics g)
  {
    g.drawArc(100, 75, 150, 100, 160, 300);
    g.fillArc(100, 200, 150, 100, 0, 250); 
  }


  /**
     Store the last selection so the paint routine can
     dispatch a call to the appropriate helper routine for
     drawing the right shape examples.
  */
  String selection;
}


