import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;

/**
   A class that demonstrates use of Graphics translation.
   This program draws an image whose top left corner is
   located at the position where the user clicks the mouse
   button.
*/
class TranslationDemo extends JPanel
{
  /**
     No-arg constructor.  Creates the listener for the mouse
     clicks, and initially loads the image.
  */
  public TranslationDemo()
  {
    createListeners();
    loadImage("../images/tree-lined-path.gif");
  }


  /**
     Creates the mouse listener that handles the user mouse
     clicks.  Upon a user click, the listener determines the
     location of the click and translates the graphics
     origin to that point.
  */
  private void createListeners()
  {
    // An anonymous inner class that handles the mouse clicks.
    //
    MouseListener mL = new MouseAdapter()
      {
	public void mouseClicked(MouseEvent e)
	  {
	    // Determine where the user clicked the mouse.
	    Point click = e.getPoint();

	    Point lastPosition = origin;

	    // Save the click location.  The paint method uses
	    // this location to redraw the image in the last
	    // location specified by a mouse click in response
	    // to the window being hidden and then raised.
	    //
	    origin = click;

	    // Get a new graphics; we don't have one yet.
	    Graphics g = getGraphics();

// 	    g.setXORMode(Color.white);
	    g.drawImage(image, lastPosition.x, lastPosition.y, 
			TranslationDemo.this);

// 	    g.setPaintMode();

	    // Clear the whole applet so we don't have residue
	    // from previous images drawn in other positions.
 	    g.clearRect(0, 0, getWidth(), getHeight());

	    // Translate the origin of the Graphics to the point
	    // clicked by the user in the applet's window.
	    g.translate(click.x, click.y);

	    // The image will now be drawn from the new origin,
	    // which is still referred to as the point (0, 0).
	    //
	    g.drawImage(image, 0, 0, TranslationDemo.this);

	    // Dispose of the graphics that we created.
	    //
	    g.dispose();
	  }
      };
    addMouseListener(mL);
  }


  private void loadImage(String path)
  {
    image = Toolkit.getDefaultToolkit().getImage(path);
  }

  public Dimension getPreferredSize()
  {
    return new Dimension(400, 300);
  }


  /**
     Paints the image.  This call displays the image the
     first time the applet is started.  Also, if the applet
     window is hidden and then raised, this method will
     redraw the applet at the location it was last drawn in
     response to last user mouse click.

     @param g the graphics created by the AWT and passed in.  
  */
  public void paint(Graphics g)
  {
    g.clearRect(0, 0, getWidth(), getHeight());
    g.drawImage(image, origin.x, origin.y, this);
  }


  public static void main(String [] args)
  {
    TranslationDemo app = new TranslationDemo();
    JFrame f = new JFrame("Translation Demo");
    WindowListener wl = new WindowAdapter()
      {
	public void windowClosing(WindowEvent e)
	  {
	    e.getWindow().dispose();
	    System.exit(0);
	  }
      };
    f.addWindowListener(wl);
    f.getContentPane().add(app);
    f.pack();
    f.setVisible(true);
  }

  // The image to display.
  Image image;

  // Initializes a new Point which specifies the location of
  // the top left corner where the image should first be
  // drawn.  
  Point origin = new Point(30, 20);
}
