import java.awt.Choice;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;

/**
   A class that creates two juxtaposed AWT List components.
   The left-hand list only supports single selection, that
   is, the selection of only a single item at a time.  The
   right-hand list supports multiple selection, the
   selection of multiple items at once.  Again, no
   getPreferredSize() method is needed.
*/
public class Lists extends Panel
{
  /**
     No-arg constructor.
  */
  public Lists()
  {
    Font f18 = new Font("SansSerif", Font.PLAIN, 18);
    Font f14 = new Font("SansSerif", Font.PLAIN, 14);

    Label single = new Label("A single selection list");
    single.setFont(f14);

    // The single selection list.  The boolean argument specifies
    // single selection.
    //
    List singleList = new List(6, false);
    singleList.setFont(f14);

    singleList.add(new String("Ludwig von Beethoven"));
    singleList.add(new String("Lee Ritenaur"));
    singleList.add(new String("Thelonious Monk"));
    singleList.add(new String("Elton John"));
    singleList.add(new String("Julio Eglesias"));
    singleList.add(new String("Benny Goodman"));

    Label multiple = new Label("A multiple selection list");
    multiple.setFont(f14);

    // The multiple selection list.  An alternative way to
    // specify the selection model is by calling the
    // setMultiplMode method.
    //
    List multipleList = new List(6);
    multipleList.setMultipleMode(true);
    multipleList.setFont(f14);

    multipleList.add(new String("Classical"));
    multipleList.add(new String("Pop"));
    multipleList.add(new String("Rock"));
    multipleList.add(new String("Swing"));
    multipleList.add(new String("Jazz"));

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    gbl.setConstraints(single, gbc);
    add(single);

    gbc.insets = new Insets(5, 5, 5, 5);
    gbc.gridx = GridBagConstraints.RELATIVE;
    gbl.setConstraints(multiple, gbc);
    add(multiple);

    gbc.gridx = 0;
    gbc.gridy = GridBagConstraints.RELATIVE;
    gbl.setConstraints(singleList, gbc);
    add(singleList);

    gbc.gridx = GridBagConstraints.RELATIVE;
    gbc.gridy = 1;
    gbl.setConstraints(multipleList, gbc);
    add(multipleList);
  }

  public static void main(String [] args)
  {
    Lists app = new Lists();

    Frame f = new Frame("Lists");
    f.add(app);
    f.pack();
    f.setVisible(true);
  }
}
