import java.awt.*;
import javax.swing.*;

/**
   This program demonstrates how to use invisible "glue" to give
   spacing between components managed by BoxLayout layout manager.
   The container used is a Box container, which always uses a
   BoxLayout layout manager.
*/
public class GlueDemo extends JPanel
{
  public GlueDemo()
  {
    // "this" container is a JPanel so we can use any layout manager
    // we want.
    setLayout(new BorderLayout());

    // Create the Box container that holds buttons 1, 2, and 3.  The
    // Box containers must use a BoxLayout.  We can only specify the
    // orientation.
    //
    Box oneToThree = Box.createHorizontalBox();
    BoxLayout box_1_3_Layout = new BoxLayout(oneToThree, BoxLayout.X_AXIS);

    oneToThree.add(new JButton("1"));

    // Within this Box container that holds buttons 1, 2, and 3, add
    // horizontal struts between the buttons.
    //
    oneToThree.add(Box.createHorizontalStrut(15));
    oneToThree.add(new JButton("2"));
    oneToThree.add(Box.createHorizontalStrut(15));
    oneToThree.add(new JButton("3"));
    
    // Create the Box container that holds buttons 4, 5, and 6.
    //
    Box fourToSix = Box.createHorizontalBox();
    BoxLayout box_4_6_Layout = new BoxLayout(fourToSix, BoxLayout.X_AXIS);

    // Also add horizontal struts between the buttons 4, 5, and 6 of
    // this second Box container.
    //
    fourToSix.add(new JButton("4"));
    fourToSix.add(Box.createHorizontalStrut(15));
    fourToSix.add(new JButton("5"));
    fourToSix.add(Box.createHorizontalStrut(15));
    fourToSix.add(new JButton("6"));

    // Create the Box container that holds buttons 7, 8, and 9.
    //
    Box sevenToNine = Box.createHorizontalBox();
    BoxLayout box_7_9_Layout = new BoxLayout(sevenToNine, BoxLayout.X_AXIS);

    // And, finally, add horizontal struts between these buttons too.
    //
    sevenToNine.add(new JButton("7"));
    sevenToNine.add(Box.createHorizontalStrut(15));
    sevenToNine.add(new JButton("8"));
    sevenToNine.add(Box.createHorizontalStrut(15));
    sevenToNine.add(new JButton("9"));

    // Create the intermediate container that holds the three
    // containers with the buttons.  Notice that this container uses a
    // vertical layout. Vertical space (struts) will be added between
    // each row of buttons.
    //
    Box intermediatePanel = Box.createVerticalBox();
    BoxLayout intermediateLayout = new BoxLayout(intermediatePanel, 
						 BoxLayout.Y_AXIS);

    intermediatePanel.add(oneToThree);

    // Add a vertical strut between the first and second rows of
    // buttons, that is, between the first and second Box container.
    //
    intermediatePanel.add(Box.createVerticalStrut(25));
    intermediatePanel.add(fourToSix);

    // Add another vertical strut between the second and third rows of
    // buttons, that is, between the second and third Box containers.
    //
    intermediatePanel.add(Box.createVerticalStrut(25));
    intermediatePanel.add(sevenToNine);

    // Added to the CENTER of our BorderLayout by default.
    //
    add(intermediatePanel);

  }

  public static void main(String [] args)
  {
    GlueDemo app = new GlueDemo();

    JFrame f = new JFrame("Glue and Strut Demo");
    f.getContentPane().add(app);
    f.pack();
    f.setVisible(true);
  }

}
