import java.awt.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import com.sun.java.swing.*;

import com.fujitsu.fnc.nms.fenics.gui.gcf.FAbstractAction;

//import GourmetAction;


class LetsEat extends JFrame
{
  public LetsEat(String title)
  {
    super(title);

    JPanel buttonPanel = createButtonPanel();
    JPanel menuPanel = createMenuPanel();
    statusPanel = createStatusPanel();

    Container contentPane = getContentPane();
    contentPane.add(buttonPanel, BorderLayout.NORTH);
    contentPane.add(menuPanel, BorderLayout.CENTER);
    contentPane.add(statusPanel, BorderLayout.SOUTH);

    GourmetAction action = new
      GourmetAction("gourmet-meal", 
		    "Simulates eating the scrumptious meal you order.",
		    null, true, false, true, true, this, dataContext, 0, null);

    enjoy.addActionListener(action);
    enjoy.setToolTipText((String) action.getValue(FAbstractAction.FLYOVER));

    MouseListener promptMonitor = new MouseAdapter()
      {
	public void mouseEntered(MouseEvent e)
	  {
	    statusPanel.remove(status);
	    status.setText(promptText);
	    statusPanel.add(status);
	  }
      };
    enjoy.addMouseListener(promptMonitor);
  }


  protected JPanel createButtonPanel()
  {
    JPanel buttonPanel = new JPanel();
    enjoy = new JButton("Enjoy meal");

    buttonPanel.add(enjoy);

    return buttonPanel;
  }


  protected JPanel createMenuPanel()
  {
    JPanel menuPanel = new JPanel();

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    menuPanel.setLayout(gbl);

    JLabel appetizerLabel = new JLabel("Appetizers");
    constraints.insets = new Insets(0, 6, 0, 0);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gbl.setConstraints(appetizerLabel, constraints);

    JLabel soupLabel = new JLabel("Soups");
    gbl.setConstraints(soupLabel, constraints);

    JLabel mainCourseLabel = new JLabel("Main Courses");
    gbl.setConstraints(mainCourseLabel, constraints);

    JLabel dessertLabel = new JLabel("Desserts");
    gbl.setConstraints(dessertLabel, constraints);

    JLabel wineLabel = new JLabel("Wines");
    gbl.setConstraints(wineLabel, constraints);

    menuPanel.add(appetizerLabel);
    menuPanel.add(soupLabel);
    menuPanel.add(mainCourseLabel);
    menuPanel.add(dessertLabel);
    menuPanel.add(wineLabel);

    JScrollPane appetizers = createAppetizerMenu();
    JScrollPane soups = createSoupMenu();
    JScrollPane mainCourses = createMainCourseMenu();
    JScrollPane desserts = createDessertMenu();
    JScrollPane wineList = createWineList();

    constraints.gridx = 0;
    constraints.gridy = GridBagConstraints.RELATIVE;
    constraints.insets = new Insets(10, 5, 10, 5);
    constraints.anchor = GridBagConstraints.CENTER;
    constraints.fill = GridBagConstraints.VERTICAL;
    menuPanel.add(appetizers, constraints);

    constraints.gridx = GridBagConstraints.RELATIVE;
    constraints.gridy = 1;
    menuPanel.add(soups, constraints);
    menuPanel.add(mainCourses, constraints);
    menuPanel.add(desserts, constraints);
    menuPanel.add(wineList, constraints);

    return menuPanel;
  }


  protected JScrollPane createAppetizerMenu()
  {
    JScrollPane sp = new JScrollPane(new JList(appetizerChoices),
				     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setPreferredSize(new Dimension(125, 200));
    return sp;
  }


  protected JScrollPane createSoupMenu()
  {
    JScrollPane sp = new JScrollPane(new JList(soupChoices),
				     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setPreferredSize(new Dimension(125, 200));
    return sp;
  }


  protected JScrollPane createMainCourseMenu()
  {
    JScrollPane sp = new JScrollPane(new JList(mainMenuChoices),
				     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setPreferredSize(new Dimension(125, 200));
    return sp;
  }


  protected JScrollPane createDessertMenu()
  {
    JScrollPane sp = new JScrollPane(new JList(dessertChoices),
				     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setPreferredSize(new Dimension(125, 200));
    return sp;
  }


  protected JScrollPane createWineList()
  {
    JScrollPane sp = new JScrollPane(new JList(wineChoices),
				     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				     JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setPreferredSize(new Dimension(125, 200));
    return sp;
  }


  protected JPanel createStatusPanel()
  {
    statusPanel = new JPanel();
    statusPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

    status = new JLabel(promptText);
    status.setHorizontalAlignment(JLabel.LEFT);
    statusPanel.add(status);

    return statusPanel;
  }


  public static void main(String [] args)
  {
    LetsEat app = new LetsEat("Action command test program");

    WindowListener wL = new WindowAdapter() 
      {
	public void windowClosing(WindowEvent e)
	  {
	    ((Window) e.getSource()).dispose();
	    System.exit(0);
	  }
      };
    app.addWindowListener(wL);
			
    app.pack();
    app.show();
  }


  JButton enjoy;

  String promptText = "Your waiter awaits your meal selection.  Select one item from each category and click the \"Enjoy\" button";

  JLabel status;
  JPanel statusPanel;

  String [] appetizerChoices = {"Bread", "Dolma", "Dumplings", "Falafal", 
				"Kimchi", "Green salad", "Tappas", 
				"Yorkshire pudding", "Peanuts"};

  String [] soupChoices = {"Cream of Broccoli", "Cream of Corn", 
			   "Sizzling rice", "Wonton", 
			   "New England clam chowder", 
			   "New York clam chowder"};

  String [] mainMenuChoices = {"Bouillabaise", "Chateaubriand", "Coq au Vin", 
			       "Gyro", "Hamburger", "Happy meal", 
			       "Fettuccini a Firenze", "Salmon Almondine", 
			       "Shish Kabab"};

  String [] dessertChoices = {"Baklava", "Chocolate truffles", "Mille feuille",
			      "Black Forest chocolate cake", "Mochi", 
			      "Red bean and tapioca", "Sara Lee pound cake", 
			      "Spumoni ice cream", "Tang Yuen"};

  String [] wineChoices = {"St. Emilion Grand Cru, 1963", 
			   "Champagne Chateau Veuve Cliquot, 1978",
			   "Chateau Rothschild Vin Rouge, 1958", 
			   "Glen-Ellen Select, 1998", 
			   "Gallo premium jug"};

  Object [] dataContext = {appetizerChoices, soupChoices, mainMenuChoices,
			   dessertChoices, wineChoices};
}



