import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

class TreeEventDemo extends JPanel
{

  public TreeEventDemo()
  {
    TreeExpansionEventMonitor expansionMonitor = new
      TreeExpansionEventMonitor();

    TreeModelListener treeModelEventMonitor = new
      TreeModelEventMonitor();

    DefaultMutableTreeNode travelPlaces = new 
      DefaultMutableTreeNode("Favorite Places to Travel");

    JTree tree = new JTree(travelPlaces);

    // Add a TreeExpansionEvent listener.
    //
    tree.addTreeExpansionListener(expansionMonitor);

    DefaultTreeModel treeModel = (DefaultTreeModel) tree.getModel();

    // Add a TreeModelEvent listener.  
    // Notice we add the listener to our tree's model, not to the
    // tree itself.  Only TreeModel instances can fire this event,
    // not trees.
    // We add this listener before we add the elements to the tree
    // so it will receive events when the elements are added.
    //
    treeModel.addTreeModelListener(treeModelEventMonitor);

    // Create sub-menu "France"
    //
    DefaultMutableTreeNode france = new 
      DefaultMutableTreeNode("France");

    DefaultMutableTreeNode paris = new 
      DefaultMutableTreeNode("Paris");

    DefaultMutableTreeNode lyon = new 
      DefaultMutableTreeNode("Lyon");

    DefaultMutableTreeNode stMalo = new 
      DefaultMutableTreeNode("St. Malo");

    // Force a TreeModel event to occur.
    //
    france.add(paris);
    treeModel.reload();

    // Force a TreeModel event to occur.
    //
    france.add(lyon);
    treeModel.reload();

    // Force a TreeModel event to occur.
    //
    france.add(stMalo);
    treeModel.reload();

    travelPlaces.add(france);

    // Create sub-menu "Indian Ocean"
    //
    DefaultMutableTreeNode indianOcean = new 
      DefaultMutableTreeNode("Indian Ocean Islands");

    DefaultMutableTreeNode maldives = new 
      DefaultMutableTreeNode("Maldives");

    DefaultMutableTreeNode mauritius = new 
      DefaultMutableTreeNode("Mauritius");

    DefaultMutableTreeNode seychelles = new 
      DefaultMutableTreeNode("Seychelles");

    indianOcean.add(maldives);
    indianOcean.add(mauritius);
    indianOcean.add(seychelles);

    travelPlaces.add(indianOcean);

    // Create sub-menu U.S.
    //
    DefaultMutableTreeNode us = new 
      DefaultMutableTreeNode("United States");

    travelPlaces.add(us);

    DefaultMutableTreeNode la = new 
      DefaultMutableTreeNode("Los Angeles");

    DefaultMutableTreeNode nb = new 
      DefaultMutableTreeNode("Newport Beach");

    DefaultMutableTreeNode sd = new 
      DefaultMutableTreeNode("San Diego");

    DefaultMutableTreeNode boston = new 
      DefaultMutableTreeNode("Pt. Arina");

    // Force a TreeModelEvent to occur.
    //
    treeModel.reload();

    us.add(la);
    us.add(nb);
    us.add(sd);
    us.add(boston);

    // Create sub-menu U.S. -> National Parks
    //
    DefaultMutableTreeNode parks = new 
      DefaultMutableTreeNode("National Parks");

    DefaultMutableTreeNode yosemite = new 
      DefaultMutableTreeNode("Yosemite");

    DefaultMutableTreeNode yellowstone = new 
      DefaultMutableTreeNode("Yellowstone");

    DefaultMutableTreeNode shasta = new 
      DefaultMutableTreeNode("Mt. Shasta");

    parks.add(yosemite);
    parks.add(yellowstone);
    parks.add(shasta);

    us.add(parks);

    // Add a TreeSelectionListener that listens for changes in the
    // selection of tree nodes or leaves.  We can add this last 
    // because this listener only listens for user selections.  
    //
    TreeSelectionListener treeSelectionEventMonitor = new
      TreeSelectionEventMonitor();
    tree.addTreeSelectionListener(treeSelectionEventMonitor);

    // Add a TreeWillExpandListener.
    // 
    //
    TreeWillExpandListener treeWillExpandMonitor = new
      TreeWillExpandEventMonitor();
    tree.addTreeWillExpandListener(treeWillExpandMonitor);

    tree.setPreferredSize(new Dimension(250, 360));
    tree.setFont(new Font("SansSerif", Font.PLAIN, 16));

    JScrollPane treePane = new JScrollPane(tree);
    treePane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    treePane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
//     treePane.setPreferredSize(new Dimension(300, 420));

    add(treePane);
  }


  public static void main(String [] args)
  {
    TreeEventDemo app = new TreeEventDemo();

    JFrame f = new JFrame("Tree Event Demo");
    WindowListener wL = new WindowAdapter()
      {
        public void windowClosing(WindowEvent e)
          {
            e.getWindow().dispose();
            System.exit(0);
          }
      };
    f.addWindowListener(wL);

    f.getContentPane().add(app);
    f.pack();
    f.setVisible(true);
  }
}


class TreeExpansionEventMonitor implements TreeExpansionListener
{
  public void treeCollapsed(TreeExpansionEvent e)
  {
    TreePath path = e.getPath();

    System.out.println("TreeExpansionEvent: Tree collapsed.");
    System.out.println("---> " + path.toString());
    System.out.println();
  }

  public void treeExpanded(TreeExpansionEvent e)
  {
    TreePath path = e.getPath();

    System.out.println("TreeExpansionEvent: Tree expanded.");
    System.out.println("Number of path elements = " + path.getPathCount());
    System.out.println("---> " + path.toString());
    System.out.println();
  }
}


class TreeSelectionEventMonitor implements TreeSelectionListener
{
  public void valueChanged(TreeSelectionEvent e)
  {
    System.out.println("TreeSelectionEvent:");
    System.out.println("New lead selection path = ");
    System.out.println("---> " + e.getNewLeadSelectionPath().toString());
    System.out.println();

    TreePath path = e.getOldLeadSelectionPath();
    if (path != null)
      {
        System.out.println("Old lead selection path = ");
        System.out.println("---> " + path.toString());
        System.out.println();
      }
  }
}


class TreeWillExpandEventMonitor implements TreeWillExpandListener
{
  public void treeWillCollapse(TreeExpansionEvent e)
  {
    System.out.println("TreeExpansionEvent: Tree will collapse.");
    System.out.println("---> " + e.getPath().toString());
    System.out.println();
  }


  public void treeWillExpand(TreeExpansionEvent e)
  {
    System.out.println("TreeExpansionEvent: Tree will expand.");
    System.out.println("---> " + e.getPath().toString());
    System.out.println();
  }
}


class TreeModelEventMonitor implements TreeModelListener
{
  public void treeNodesChanged(TreeModelEvent e)
  {
    TreePath path = e.getTreePath();

    System.out.println("TreeModelEvent: Tree nodes changed");
    printTreePathInfo(path);
  }


  public void treeNodesInserted(TreeModelEvent e)
  {
    TreePath path = e.getTreePath();
    System.out.println("TreeModelEvent: Tree nodes inserted.");
    printTreePathInfo(path);
  }


  public void treeNodesRemoved(TreeModelEvent e)
  {
    TreePath path = e.getTreePath();

    System.out.println("TreeModelEvent: Tree nodes removed.");
    printTreePathInfo(path);
  }


  public void treeStructureChanged(TreeModelEvent e)
  {
    TreePath path = e.getTreePath();

    System.out.println("TreeModelEvent; Tree structure changed.");
    printTreePathInfo(path);
  }


  void printTreePathInfo(TreePath path)
  {
    System.out.println("Number of path elements = " + path.getPathCount());
    System.out.println("---> " + path.toString());
    System.out.println();
  }
}



