import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;


public class InternalFrameEventDemo extends JPanel
{
  public InternalFrameEventDemo()
  {
    JInternalFrame iF1 = new JInternalFrame("Internal Frame 1", true, true,
					    true, true);
    iF1.setPreferredSize(new Dimension(350, 90));
    Container contentPane = iF1.getContentPane();
    contentPane.setLayout(new FlowLayout());
    contentPane.add(new JButton("Button A"));
    contentPane.add(new JButton("Button B"));
    contentPane.add(new JButton("Button C"));

    JInternalFrame iF2 = new JInternalFrame("Internal Frame 2", true, true,
					    true, true);
    iF2.setPreferredSize(new Dimension(350, 90));
    contentPane = iF2.getContentPane();
    contentPane.setLayout(new FlowLayout());
    contentPane.add(new JButton("Button D"));
    contentPane.add(new JButton("Button E"));
    contentPane.add(new JButton("Button F"));

    JInternalFrame iF3 = new JInternalFrame("Internal Frame 3", true, true,
					    true, true);
    iF3.setPreferredSize(new Dimension(350, 90));
    contentPane = iF3.getContentPane();
    contentPane.setLayout(new FlowLayout());
    contentPane.add(new JButton("Button G"));
    contentPane.add(new JButton("Button H"));
    contentPane.add(new JButton("Button I"));

    // Create an internal frame event listener.
    InternalFrameListener fL = new InternalFrameMonitor();

    // Add this same listener as a listener on all three of
    // our internal frames.  
    //
    iF1.addInternalFrameListener(fL);
    iF2.addInternalFrameListener(fL);
    iF3.addInternalFrameListener(fL);

    add(iF1);
    add(iF2);
    add(iF3);
  }

  public Dimension getPreferredSize()
  {
    return new Dimension(450, 400);
  }

  public static void main(String [] args)
  {
    InternalFrameEventDemo app = new InternalFrameEventDemo();

    JFrame f = new JFrame("Internal Frame Event Demo");
    WindowListener wL = new WindowAdapter()
      {
	public void windowClosing(WindowEvent e)
	  {
	    e.getWindow().dispose();
	    System.exit(0);
	  }
      };
    f.addWindowListener(wL);

    f.getContentPane().add(app);
    f.pack();
    f.setVisible(true);
  }


}


class InternalFrameMonitor implements InternalFrameListener
{
  public void internalFrameActivated(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println(e.paramString());
//     System.out.println("Frame \"" + f.getTitle() + "\" activated.");
  }

  public void internalFrameClosed(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println("Frame \"" + f.getTitle() + "\" closed.");
    System.out.println();
  }

  public void internalFrameClosing(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println("Frame \"" + f.getTitle() + "\" closing.");
    System.out.println();
  }

  public void internalFrameDeactivated(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println("Frame \"" + f.getTitle() + "\" deactivated.");
    System.out.println();
  }

  public void internalFrameDeiconified(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println("Frame '" + f.getTitle() + "' de-iconified.");
    System.out.println();
  }

  public void internalFrameIconified(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println("Frame '" + f.getTitle() + "' iconified.");
    System.out.println();
  }

  public void internalFrameOpened(InternalFrameEvent e)
  {
    JInternalFrame f = (JInternalFrame) e.getSource();
    System.out.println("Frame '" + f.getTitle() + "' opened.");
    System.out.println();
  }

}



